﻿package MT::Plugin::FlattingTemplate;
use strict;
use MT;
use MT::Plugin;
use MT::Template;
use FlattingTemplate::TemplateDependence;

require MT::Plugin;

our $VERSION = '0.1';
our $SCHEMA_VERSION = '0.1';

use base qw( MT::Plugin );

@MT::Plugin::FlattingTemplate::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::FlattingTemplate({
    id => 'FlattingTemplate',
    key => 'flattingtemplate',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    name => 'Flatting Template',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    'version' => $VERSION,
    'schema_version' => $SCHEMA_VERSION,
    l10n_class => 'FlattingTemplate::L10N',
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'template' => {
                'modularized_text' => 'text',
            },
            'templatedependence' => 'FlattingTemplate::TemplateDependence',
        },
        callbacks => {
            'MT::App::CMS::template_param.edit_template'
                => {
                    handler => \&_edit_template,
                    priority => 1,
                },
            'cms_post_save.template'
                => {
                    handler => \&_smash_and_save,
                    priority => 9,
                },
        },
        applications => {
            cms => {
                list_actions => {
                    template => {
                        template_flatting => {
                            label => $plugin->translate('Flatting Template(s)'),
                            code => \&_smash,
                            permissions => 'can_edit_templates',
                        },
                        template_2module => {
                            label => $plugin->translate('Modularize Template(s)'),
                            code => \&_modularize,
                            permissions => 'can_edit_templates',
                        },
                    },
                },
            },
        },
   });
}

sub _smash {
    my $app = shift;
    my @id = $app->param('id') or return $app->errtrans('Invalid request.');
    my $user = $app->user;
    require MT::Template;
    foreach my $id (@id) {
        next unless $id;
        my $temp = MT::Template->load($id);
        next unless $temp;
        my $old_text = $temp->text;
        my $perms = $user->permissions($temp->blog_id);
        next unless $perms && $perms->can_edit_templates;
        if (MT::Plugin::TemplateHammer::smash_it($temp)) {
            $temp->modularized_text($old_text);
            $temp->save;
        }
    }
    $app->call_return;
}

sub _modularize {
    my $app = shift;
    my @id = $app->param('id') or return $app->errtrans('Invalid request.');
    my $user = $app->user;
    require MT::Template;
    foreach my $id (@id) {
        next unless $id;
        my $temp = MT::Template->load($id);
        next unless $temp;
        my $modularized_text = $temp->modularized_text;
        my $perms = $user->permissions($temp->blog_id);
        next unless $perms && $perms->can_edit_templates;
        if ($modularized_text) {
            $temp->text($modularized_text);
            $temp->modularized_text('');
            $temp->save
                or return $app->trans_error('Error saving template: [_1]', $temp->errstr);
        }
    }
    $app->call_return;
}

sub _edit_template {
    my ($cb, $app, $param) = @_;
    my $q = $app->param;
    my $id = $q->param('id');
    my $saved = $q->param('saved');
    if ($id) {
        my $temp = MT::Template->load( { id => $id } );
        if ($temp->modularized_text) {
            $temp->text($temp->modularized_text);
            my $clone = $temp->clone();
            $param->{'text'} = $temp->modularized_text;
            my $modules = $clone->getElementsByTagName('Include');
            my %saved_module;
            if ( $modules ) {
                my @modules; my @widgets; my %loaded_tmpl;
                foreach my $tag ( @$modules ) {
                    my $include = {};
                    my $module; my $type; 
                    if ($tag->[1]->{module}) {
                        $module = $tag->[1]->{module};
                        $type = 'custom';
                    } elsif ($tag->[1]->{widget}) {
                        $module = $tag->[1]->{widget};
                        $type = 'widget';
                    }
                    unless ( (! $module ) || ( exists $loaded_tmpl{$type}{$module} ) ) {
                        $include->{include_module} = $module;
                        $loaded_tmpl{$type}{$module} = 1;
                        my $tmpl = MT::Template->load( {
                                                            blog_id => $clone->blog_id,
                                                            name    => $module,
                                                            type    => $type,
                                                       } );
                        if ($tmpl) {
                            $include->{include_link} = $app->mt_uri( mode => 'view',
                                                                     args => {
                                                                         blog_id => $tmpl->blog_id,
                                                                         '_type' => 'template',
                                                                         id      => $tmpl->id
                                                                     }
                                                                   );
                            if ($saved) {
                                my $dependence = FlattingTemplate::TemplateDependence->get_by_key( { 
                                                                                                     parent_id   => $temp->id,
                                                                                                     template_id => $tmpl->id,
                                                                                                    }
                                                                                                 );
                                $dependence->template_type($type);
                                $dependence->save
                                    or return $app->trans_error('Error saving template dependence: [_1]', $dependence->errstr);
                                $saved_module{$type}{$tmpl->name} = 1;
                            }

                        } else {
                            $include->{create_link} = $app->mt_uri(
                                                                    mode => 'view',
                                                                    args => {
                                                                    blog_id => $clone->blog_id,
                                                                        '_type' => 'template',
                                                                        type    => $type,
                                                                        name    => $module,
                                                                    }
                                                                );
                        }
                        if ($type eq 'widget') {
                            push @widgets, $include;
                        } else {
                            push @modules, $include;
                        }
                    }
                }
                if (@modules || @widgets) {
                    $param->{'have_includes'} = 1;
                }
                $param->{include_loop} = \@modules if @modules;
                $param->{widget_loop} = \@widgets if @widgets;
            }
            my @depes = FlattingTemplate::TemplateDependence->load( { parent_id => $id } );
            foreach my $dep (@depes) {
                my $type = $dep->template_type;
                my $mod = MT::Template->load( { id => $dep->template_id } );
                my $name = $mod->name;
                unless ( exists $saved_module{$type}{$name} ) {
                    $dep->remove
                        or return $app->trans_error('Error removing template dependence: [_1]', $dep->errstr);
                }
            }
        }
    }
}

sub _smash_and_save {
    my ($eh, $app, $temp, $original) = @_;
    my $q = $app->param;
    if ($temp->id) {
        my @templates = MT::Template->load( undef,
                                    { join => [ 'FlattingTemplate::TemplateDependence', 'parent_id',
                                              { template_id => $temp->id },
                                              { unique => 1 }
                                               ],} );
        if (@templates) {
            foreach my $template (@templates) {
                my $modularized_text = $template->modularized_text;
                my $clone = $template->clone();
                $clone->text($modularized_text);
                if ( MT::Plugin::TemplateHammer::smash_it( $clone ) ) {
                    $clone->modularized_text($template->modularized_text);
                    $clone->save
                        or return $app->trans_error('Error saving template: [_1]', $clone->errstr);
                }
            }
        }
        my $old_text = $temp->text;
        if (MT::Plugin::TemplateHammer::smash_it($temp)) {
            $temp->modularized_text($old_text);
        } else {
            $temp->modularized_text('');
        }
        $temp->save
            or return $app->trans_error('Error saving template: [_1]', $temp->errstr);
    }
}

1;